from senkalib.chain.transaction import Transaction
from senkalib.chain.transaction_generator import TransactionGenerator
from senkalib.senka_setting import SenkaSetting
from senkalib.chain.osmosis.osmosis_transaction import OsmosisTransaction
from typing import List
import requests

class OsmosisTransactionGenerator(TransactionGenerator):
  chain = 'osmosis'

  @classmethod
  def get_transactions(cls, settings:SenkaSetting, address:str, timerange:dict = None, blockrange:dict = None) -> List[Transaction]:
    osmosis_transactions = []
    num_transactions = 50
    startblock = int(blockrange['startblock']) if blockrange is not None and 'startblock' in blockrange and type(blockrange['startblock']) is int else 0
    endblock = int(blockrange['endblock']) if blockrange is not None and 'endblock' in blockrange and type(blockrange['endblock']) is int else None

    while num_transactions >= 50:
      transactions = OsmosisTransactionGenerator.get_txs(address, startblock)
      num_transactions = len(transactions)
      for transaction in transactions:
        startblock = transaction['header']['id']
        if endblock is not None and startblock <= endblock:
          return osmosis_transactions
        osmosis_transactions.append(OsmosisTransaction(transaction))


    return osmosis_transactions

  @classmethod
  def get_txs(cls, address, startblock):
    response = requests.get(
        'https://api-osmosis.cosmostation.io/v1/account/new_txs/%s' % address,
        params={'from': startblock, 'limit': 50})
    transactions = response.json()
    return transactions


import re
import unittest
from unittest.mock import *
from senkalib.chain.bsc.bsc_transaction_generator import BscTransactionGenerator
from senkalib.chain.osmosis.osmosis_transaction_generator import OsmosisTransactionGenerator
from senkalib.senka_lib import SenkaLib
from senkalib.caaj_journal import CaajJournal
import os
from decimal import Decimal

class TestSenkaLib(unittest.TestCase):
  def test_get_available_chain(self):
    with patch.object(os, 'listdir', new=TestSenkaLib.mock_listdir):
      chains = SenkaLib.get_available_chain()

      self.assertEqual(chains[0], BscTransactionGenerator)
      self.assertEqual(chains[1], OsmosisTransactionGenerator)

  @classmethod
  def mock_listdir(cls, transaction_hash):
    return ['osmosis', 'bsc']

if __name__ == '__main__':
  unittest.main()



from senkalib.chain.transaction import Transaction 
from decimal import Decimal
import datetime

class BscTransaction(Transaction):
  chain = 'bsc'

  def __init__(self, transaction_id:str, transaction_receipt:dict, timestamp:str, gasused:str, gasprice:str):
    super().__init__(transaction_id)
    self.transaction_receipt = transaction_receipt
    self.timestamp = timestamp
    self.gasused = Decimal(gasused)
    self.gasprice = Decimal(gasprice)

  def get_timestamp(self) -> str:
    return str(datetime.datetime.fromtimestamp(int(self.timestamp), datetime.timezone.utc).replace(tzinfo=None))

  def get_transaction_fee(self) -> Decimal:
    return self.gasused * self.gasprice

  def get_transaction_receipt(self) -> dict:
    return self.transaction_receipt


from abc import ABCMeta, abstractmethod
from senkalib.chain.transaction import Transaction 
from senkalib.senka_setting import SenkaSetting
from typing import List

class TransactionGenerator(metaclass=ABCMeta):
  chain = None

  @classmethod
  @abstractmethod
  def get_transactions(cls, settings:SenkaSetting, address:str, timerange:dict = None, blockrange:dict = None) -> List[Transaction]:
    pass


from typing import List
import importlib
import os
import re
from senkalib.chain.transaction_generator import TransactionGenerator
import csv
import requests
import io

from senkalib.caaj_journal import CaajJournal

class SenkaLib:
  TOKEN_OERIGINAK_IDS_URL = 'https://raw.githubusercontent.com/ca3-caaip/token_original_id/master/token_original_id.csv'

  @classmethod
  def get_available_chain(cls, blacklist = []) -> List[TransactionGenerator]:
    available_chains = []
    try:
      blacklist.append('__pycache__')
      path = '%s/chain' % os.path.dirname(__file__)
      files = os.listdir(path)
      dirs = sorted([f for f in files if os.path.isdir(os.path.join(path, f)) and f not in blacklist])
  
      for dir in dirs:
        module = importlib.import_module(f"senkalib.chain.{dir}.{dir}_transaction_generator", f"senkalib")
        transaction_generator = list(f"{dir}_transaction_generator")
        transaction_generator[0] = transaction_generator[0].upper()
        transaction_generator = "".join(transaction_generator)
        transaction_generator = re.sub("_(.)",lambda x:x.group(1).upper(), transaction_generator)
        transaction_generator = getattr(module, transaction_generator)
        available_chains.append(transaction_generator)
    except Exception as e:
      f"failed to load senkalib transaction generator: {e}"
      raise e


    return available_chains

  @classmethod
  def get_token_original_ids(cls) -> dict:
    url = cls.TOKEN_OERIGINAK_IDS_URL
    res = requests.get(url).content.decode()
    csv_reader = csv.DictReader(res.strip().splitlines())
    read_data = [row for row in csv_reader]
    return read_data



class SenkaSetting:
  def __init__(self, settings:dict):
    self.settings = settings

  def set_settings(self, settings:dict):
    self.settings = settings

  def get_settings(self) -> dict:
    return self.settings


from senkalib.chain.bsc.bsc_transaction import BscTransaction
from senkalib.chain.transaction import Transaction 
from senkalib.chain.transaction_generator import TransactionGenerator
from senkalib.senka_setting import SenkaSetting
from web3 import Web3
from bscscan import BscScan
from typing import List
import asyncio

class BscTransactionGenerator(TransactionGenerator):
  chain = 'bsc'

  @classmethod
  def get_transactions(cls, settings:SenkaSetting, address:str, timerange:dict = None, blockrange:dict = None) -> List[Transaction]:
    settings = settings.get_settings()
    startblock = blockrange['startblock'] if blockrange is not None and 'startblock' in blockrange and type(blockrange['startblock']) is int else 0
    endblock = blockrange['endblock'] if blockrange is not None and 'endblock' in blockrange and type(blockrange['endblock']) is int else 'latest'
    w3 = Web3(Web3.HTTPProvider("https://bsc-dataseed.binance.org/"))
    transactions = []
    
    page = 1
    while True:
      txs = asyncio.run(BscTransactionGenerator.get_txs(settings, address, startblock, endblock, page))
      for tx in txs:
        if '1' in tx['isError']:
          continue
        else:
          receipt = w3.eth.get_transaction_receipt(tx['hash'])
          transactions.append(BscTransaction(tx['hash'], receipt, tx['timeStamp'], tx['gasUsed'], tx['gasPrice']))

      page += 1
      if len(txs) < 10000: # bscscan api return 10000 results for each page
        break

    return transactions

  @classmethod
  async def get_txs(cls, settings:dict, address:str, arg_startblock:int = None, arg_endblock:int = None, arg_page:int = None):
    async with BscScan(settings['bscscan_key']) as bscscan:
      txs = await bscscan.get_normal_txs_by_address_paginated(address=address, startblock=arg_startblock, endblock=arg_endblock, 
        page=arg_page, offset=0, sort='asc')

    return txs


from abc import ABCMeta, abstractmethod
from senkalib.caaj_journal import CaajJournal
from senkalib.chain.transaction import Transaction
from typing import List

class CaajPlugin(metaclass=ABCMeta):
  chain = None

  @abstractmethod
  def can_handle(cls, transaction:Transaction) -> bool:
    pass

  @abstractmethod
  def get_caajs(cls, address:str, transaction:Transaction, token_original_ids:List) -> List[CaajJournal]:
    pass


from typing import List
from decimal import Decimal
import dataclasses

@dataclasses.dataclass
class CaajJournal:
  executed_at: str
  chain: str
  platform: str
  application: str
  transaction_id: str
  trade_uuid: str
  type: str
  amount: Decimal
  token_symbol: str
  token_original_id: str
  token_symbol_uuid: str
  caaj_from: str
  caaj_to: str
  comment: str



from platform import platform
import unittest
from senkalib.caaj_journal import CaajJournal
from decimal import Decimal

class TestCaajJounal(unittest.TestCase):
  def test_init(self):
    executed_at = '2022-01-12 11:11:11'
    chain = 'chain'
    platform = 'platform'
    application = 'application'
    transaction_id = '0x36512c7e09e3570dfc53176252678ee9617660550d36f4da797afba6fc55bba6'
    trade_uuid = 'bbbbbbddddddd'
    type = 'deposit'
    amount = Decimal('0.005147')
    token_symbol = 'juno'
    token_original_id = 'ibc/46B44899322F3CD854D2D46DEEF881958467CDD4B3B10086DA49296BBED94BED'
    token_symbol_uuid = '3a2570c5-15c4-2860-52a8-bff14f27a236'
    caaj_from = '0x111111111111111111111'
    caaj_to = '0x222222222222222222222'
    comment = 'hello world'

    cj = CaajJournal(executed_at, chain, platform, application, transaction_id, trade_uuid,
      type, amount, token_symbol, token_original_id, token_symbol_uuid, caaj_from, caaj_to, comment)
    assert cj.executed_at == '2022-01-12 11:11:11'
    assert cj.chain == 'chain'
    assert cj.platform == 'platform'
    assert cj.application == 'application'
    assert cj.transaction_id == '0x36512c7e09e3570dfc53176252678ee9617660550d36f4da797afba6fc55bba6'
    assert cj.trade_uuid == 'bbbbbbddddddd'
    assert cj.type == 'deposit'
    assert cj.amount == Decimal('0.005147')
    assert cj.token_symbol == 'juno'
    assert cj.token_original_id == 'ibc/46B44899322F3CD854D2D46DEEF881958467CDD4B3B10086DA49296BBED94BED'
    assert cj.token_symbol_uuid == '3a2570c5-15c4-2860-52a8-bff14f27a236'
    assert cj.caaj_from == '0x111111111111111111111'
    assert cj.caaj_to == '0x222222222222222222222'
    assert cj.comment == 'hello world'

if __name__ == '__main__':
  unittest.main()


from senkalib.chain.transaction import Transaction 
from decimal import *
from datetime import datetime as dt

getcontext().prec = 50

class OsmosisTransaction(Transaction):
  chain = 'osmosis'

  def __init__(self, transaction:dict):
    super().__init__(transaction["data"]["txhash"])
    self.transaction = transaction

  def get_timestamp(self) -> str:
    return str(dt.strptime(self.transaction['header']['timestamp'], '%Y-%m-%dT%H:%M:%SZ').replace(tzinfo=None))

  def get_transaction_fee(self) -> Decimal:
    fee_list = self.transaction['data']['tx']['auth_info']['fee']['amount']
    return Decimal('0') if len(fee_list) == 0 else Decimal(fee_list[0]['amount'])

  def get_transaction(self) -> dict:
    return self.transaction


import unittest
from unittest.mock import *
from senkalib.caaj_plugin import CaajPlugin
from senkalib.chain.transaction import Transaction
from senkalib.caaj_journal import CaajJournal
from typing import List
from decimal import Decimal

class SampleCaajPlugin(CaajPlugin):
  @classmethod
  def can_handle(cls, transaction:Transaction):
    return True

  @classmethod
  def get_caajs(cls, address:str, transaction:Transaction) -> List[CaajJournal]:
    jounal = CaajJournal('2022-01-01 00:00:00', 'chain', 'platform', 'application', '0x00000000',
      '3232-543543-5435443-543453', 'deposit', Decimal('0.005147'), 'juno', 
      'ibc/46B44899322F3CD854D2D46DEEF881958467CDD4B3B10086DA49296BBED94BED',
      '3a2570c5-15c4-2860-52a8-bff14f27a236', '0xaaaaa', '0xbbbbb', 'hello world')
    return [jounal]

class TestCaajPlugin(unittest.TestCase):
  def test_cat_handle(self):
    result = SampleCaajPlugin.can_handle('XXXXXXXXXXXXXXXXXXXXXXX')
    assert result == True

  def test_get_caajs(self):
    result = SampleCaajPlugin.get_caajs('0x1111111111111111', None)
    assert type(result[0]) == CaajJournal

if __name__ == '__main__':
  unittest.main()


from abc import ABCMeta, abstractmethod
from decimal import Decimal

class Transaction(metaclass=ABCMeta):
  def __init__(self, transaction_id:str):
    self.transaction_id = transaction_id

  def get_transaction_id(self) -> str:
    return self.transaction_id

  @abstractmethod
  def get_timestamp(self) -> str:
    pass

  @abstractmethod
  def get_transaction_fee(self) -> Decimal:
    pass